@@ -74,12 +74,6 @@ namespace settings {
   
 using camp::pair;
   
-#ifdef HAVE_LIBOSMESA  
-const bool defaultoffscreen=true;
-#else
-const bool defaultoffscreen=false;
-#endif
-
 #ifdef HAVE_GL
 const bool havegl=true;  
 #else
@@ -90,7 +84,7 @@ mode_t mask;
   
 string systemDir=ASYMPTOTE_SYSDIR;
 
-#ifndef __CYGWIN__
+#ifndef __MSDOS__
   
 bool msdos=false;
 string HOME="HOME";
@@ -103,6 +97,7 @@ string defaultPDFViewer="open";
 string defaultPDFViewer="acroread";
 #endif  
 string defaultGhostscript="gs";
+string defaultGhostscriptLibrary="/usr/lib/libgs.so";
 string defaultDisplay="display";
 string defaultAnimate="animate";
 void queryRegistry() {}
@@ -118,7 +113,8 @@ const char pathSeparator=';';
 string defaultPSViewer="cmd";
 //string defaultPDFViewer="AcroRd32.exe";
 string defaultPDFViewer="cmd";
-string defaultGhostscript="gswin32c.exe";
+string defaultGhostscript;
+string defaultGhostscriptLibrary;
 //string defaultDisplay="imdisplay";
 string defaultDisplay="cmd";
 //string defaultAnimate="animate";
@@ -130,7 +126,7 @@ const string dirsep="\\";
 // Use key to look up an entry in the MSWindows registry, respecting wild cards
 string getEntry(const string& location, const string& key)
 {
-  string path="/proc/registry/"+location+key;
+  string path="/proc/registry"+location+key;
   size_t star;
   string head;
   while((star=path.find("*")) < string::npos) {
@@ -178,20 +174,25 @@ string getEntry(const string& location, const string& key)
 // Use key to look up an entry in the MSWindows registry, respecting wild cards
 string getEntry(const string& key)
 {
-  string entry=getEntry("HKEY_CURRENT_USER/Software/",key);
-  if(entry.empty()) entry=getEntry("HKEY_LOCAL_MACHINE/SOFTWARE/",key);
+  string entry=getEntry("64/HKEY_CURRENT_USER/Software/",key);
+  if(entry.empty()) entry=getEntry("64/HKEY_LOCAL_MACHINE/SOFTWARE/",key);
+  if(entry.empty()) entry=getEntry("/HKEY_CURRENT_USER/Software/",key);
+  if(entry.empty()) entry=getEntry("/HKEY_LOCAL_MACHINE/SOFTWARE/",key);
   return entry;
 }
 
 void queryRegistry()
 {
-  string gs=getEntry("GPL Ghostscript/*/GS_DLL");
-  if(gs.empty())
-    gs=getEntry("AFPL Ghostscript/*/GS_DLL");
-  defaultGhostscript=stripFile(gs)+defaultGhostscript;
+  defaultGhostscriptLibrary=getEntry("GPL Ghostscript/*/GS_DLL");
+  if(defaultGhostscriptLibrary.empty())
+    defaultGhostscriptLibrary=getEntry("AFPL Ghostscript/*/GS_DLL");
+  
+  string gslib=stripDir(defaultGhostscriptLibrary);
+  defaultGhostscript=stripFile(defaultGhostscriptLibrary)+
+    ((gslib.empty() || gslib.substr(5,2) == "32") ? "gswin32c.exe" : "gswin64c.exe");
   if(defaultPDFViewer != "cmd")
-  defaultPDFViewer=getEntry("Adobe/Acrobat Reader/*/InstallPath/@")+"\\"+
-    defaultPDFViewer;
+    defaultPDFViewer=getEntry("Adobe/Acrobat Reader/*/InstallPath/@")+"\\"+
+      defaultPDFViewer;
   if(defaultPSViewer != "cmd")
     defaultPSViewer=getEntry("Ghostgum/GSview/*")+"\\gsview\\"+defaultPSViewer;
   string s;
@@ -213,6 +214,9 @@ char *argv0;
 
 // The verbosity setting, a global variable.
 Int verbose;
+
+// Conserve memory at the expense of speed.
+bool compact;
   
 // Colorspace conversion flags (stored in global variables for efficiency). 
 bool gray;
@@ -694,8 +698,8 @@ struct stringArraySetting : public itemSetting {
 
 struct engineSetting : public argumentSetting {
   engineSetting(string name, char code,
-               string argname, string desc,
-               string defaultValue)
+                string argname, string desc,
+                string defaultValue)
     : argumentSetting(name, code, argname, description(desc,defaultValue),
                       types::primString(), (item)defaultValue) {}
 
@@ -703,7 +707,8 @@ struct engineSetting : public argumentSetting {
     string str=optarg;
     
     if(str == "latex" || str == "pdflatex" || str == "xelatex" ||
-       str == "tex" || str == "pdftex" || str == "context" || str == "none") {
+       str == "tex" || str == "pdftex" || str == "luatex" ||
+       str == "lualatex" || str == "context" || str == "none") {
       value=str;
       return true;
     }
@@ -760,12 +765,12 @@ struct boolrefSetting : public refSetting<bool> {
                  bool Default=false)
     : refSetting<bool>(name, code, noarg, desc,
                        types::primBoolean(), ref, Default) {}
-  bool getOption() {
+  virtual bool getOption() {
     *ref=true;
     return true;
   }
   
-  option *negation(string name) {
+  virtual option *negation(string name) {
     struct negOption : public option {
       boolrefSetting &base;
 
@@ -790,6 +795,21 @@ struct boolrefSetting : public refSetting<bool> {
   }
 };
 
+struct compactSetting : public boolrefSetting {
+  compactSetting(string name, char code, string desc, bool *ref,
+                 bool Default=false)
+    : boolrefSetting(name,code,desc,ref,Default) {}
+  bool getOption() {
+    mem::compact(1);
+    return boolrefSetting::getOption();
+  }
+  
+  option *negation(string name) {
+    mem::compact(0);
+    return boolrefSetting::negation(name);
+  }
+};
+  
 struct incrementSetting : public refSetting<Int> {
   incrementSetting(string name, char code, string desc, Int *ref)
     : refSetting<Int>(name, code, noarg, desc,
@@ -1088,6 +1108,8 @@ void initSettings() {
                             "Embed 3D PRC graphics in PDF output", true));
   addOption(new boolSetting("toolbar", 0,
                             "Show 3D toolbar in PDF output", true));
+  addOption(new boolSetting("axes3", 0,
+                            "Show 3D axes in PDF output", true));
   addOption(new realSetting("render", 0, "n",
                             "Render 3D graphics using n pixels per bp (-1=auto)",
                             havegl ? -1.0 : 0.0));
@@ -1096,7 +1118,7 @@ void initSettings() {
   addOption(new IntSetting("multisample", 0, "n",
                            "Multisampling width for screen images", 4));
   addOption(new boolSetting("offscreen", 0,
-                            "Use offscreen rendering",defaultoffscreen));
+                            "Use offscreen rendering",false));
   addOption(new boolSetting("twosided", 0,
                             "Use two-sided 3D lighting model for rendering",
                             true));
@@ -1140,7 +1162,7 @@ void initSettings() {
   addOption(new boolSetting("keepaux", 0,
                             "Keep intermediate LaTeX .aux files"));
   addOption(new engineSetting("tex", 0, "engine",
-                              "latex|pdflatex|xelatex|tex|pdftex|context|none",
+                              "latex|pdflatex|xelatex|tex|pdftex|luatex|lualatex|none",
                               "latex"));
 
   addOption(new boolSetting("twice", 0,
@@ -1193,9 +1215,9 @@ void initSettings() {
                              &startpath));
   
 #ifdef USEGC  
-  addOption(new boolrefSetting("compact", 0,
+  addOption(new compactSetting("compact", 0,
                                "Conserve memory at the expense of speed",
-                               (bool *) &GC_dont_expand));
+                               &compact));
   addOption(new divisorOption("divisor", 0, "n",
                               "Garbage collect using purge(divisor=n) [2]"));
 #endif  
@@ -1209,8 +1231,8 @@ void initSettings() {
 
   addOption(new boolSetting("wait", 0,
                             "Wait for child processes to finish before exiting"));
-  // Be interactive even in a pipe
-  addOption(new boolSetting("interactive", 0, ""));
+  addOption(new IntSetting("inpipe", 0, "n","",-1));
+  addOption(new IntSetting("outpipe", 0, "n","",-1));
   addOption(new boolSetting("exitonEOF", 0, "Exit interactive mode on EOF",
                             true));
                             
@@ -1276,6 +1298,7 @@ void initSettings() {
   addOption(new envSetting("pdfviewer", defaultPDFViewer));
   addOption(new envSetting("psviewer", defaultPSViewer));
   addOption(new envSetting("gs", defaultGhostscript));
+  addOption(new envSetting("libgs", defaultGhostscriptLibrary));
   addOption(new envSetting("texpath", ""));
   addOption(new envSetting("texcommand", ""));
   addOption(new envSetting("dvips", "dvips"));
@@ -1299,17 +1322,19 @@ void initSettings() {
 int numArgs() { return argCount; }
 char *getArg(int n) { return argList[n]; }
 
-void setInteractive() {
+void setInteractive()
+{
   if(numArgs() == 0 && !getSetting<bool>("listvariables") && 
      getSetting<string>("command").empty() &&
-     (isatty(STDIN_FILENO) || getSetting<bool>("interactive")))
+     (isatty(STDIN_FILENO) || getSetting<Int>("inpipe") >= 0))
     interact::interactive=true;
   
   historyname=getSetting<bool>("localhistory") ? 
     (string(getPath())+dirsep+"."+suffix+"_history") : (initdir+"/history");
 }
 
-bool view() {
+bool view()
+{
   if (interact::interactive)
     return getSetting<bool>("interactiveView");
   else
@@ -1317,7 +1342,8 @@ bool view() {
       (numArgs() == 1 || getSetting<bool>("multipleView"));
 }
 
-bool trap() {
+bool trap()
+{
   if (interact::interactive)
     return !getSetting<bool>("interactiveMask");
   else
@@ -1351,9 +1377,9 @@ string lookup(const string& symbol)
 
 void initDir() {
   if(getSetting<string>("sysdir").empty()) {
-    string s=lookup("SELFAUTOPARENT");
+    string s=lookup("TEXMFMAIN");
     if(s.size() > 1) {
-      string texmf=s+dirsep+"texmf"+dirsep;
+      string texmf=s+dirsep;
       docdir=texmf+"doc"+dirsep+"asymptote";
       Setting("sysdir")=texmf+"asymptote";
       s=lookup("ASYMPTOTE_HOME");
@@ -1368,7 +1394,7 @@ void initDir() {
   if(initdir.empty())
     initdir=Getenv(HOME.c_str(),msdos)+dirsep+"."+suffix;
   
-#ifdef __CYGWIN__  
+#ifdef __MSDOS__  
   mask=umask(0);
   if(mask == 0) mask=0027;
   umask(mask);
@@ -1418,51 +1444,42 @@ void SetPageDimensions() {
   }
 }
 
-bool xe(const string& texengine) {
+bool xe(const string& texengine)
+{
   return texengine == "xelatex";
 }
 
-bool context(const string& texengine) {
+bool context(const string& texengine)
+{
   return texengine == "context";
 }
 
-bool pdf(const string& texengine) {
+bool pdf(const string& texengine)
+{
   return texengine == "pdflatex" || texengine == "pdftex" || xe(texengine) ||
-    context(texengine);
+    texengine == "luatex" || texengine == "lualatex" || context(texengine);
 }
 
-bool latex(const string& texengine) {
+bool latex(const string& texengine)
+{
   return texengine == "latex" || texengine == "pdflatex" || 
-    texengine == "xelatex";
+    texengine == "xelatex" || texengine == "lualatex";
 }
 
-string nativeformat() {
+string nativeformat()
+{
   return pdf(getSetting<string>("tex")) ? "pdf" : "eps";
 }
 
-string defaultformat() {
+string defaultformat()
+{
   string format=getSetting<string>("outformat");
   return (format.empty()) ? nativeformat() : format;
 }
 
-// Begin TeX put command.
-const char *beginput(const string& texengine) {
-  if(context(texengine))
-    return "\\put";
-  else
-    return "\\put(#1,#2)";
-}
-
-// End TeX put command.
-const char *endput(const string& texengine) {
-  if(context(texengine)) 
-    return " at #1 #2"; 
-  else
-    return "";
-}
-
 // TeX special command to set up currentmatrix for typesetting labels.
-const char *beginlabel(const string& texengine) {
+const char *beginlabel(const string& texengine)
+{
   if(pdf(texengine))
     return xe(texengine) ? "\\special{pdf:literal q #5 0 0 cm}" :
       "\\special{pdf:q #5 0 0 cm}";
@@ -1472,7 +1489,8 @@ const char *beginlabel(const string& texengine) {
 }
 
 // TeX special command to restore currentmatrix after typesetting labels.
-const char *endlabel(const string& texengine) {
+const char *endlabel(const string& texengine)
+{
   if(pdf(texengine))
     return xe(texengine) ? "\\special{pdf:literal Q}" : "\\special{pdf:Q}";
   else
@@ -1480,7 +1498,8 @@ const char *endlabel(const string& texengine) {
 }
 
 // TeX macro to typeset raw postscript code
-const char *rawpostscript(const string& texengine) {
+const char *rawpostscript(const string& texengine)
+{
   if(pdf(texengine))
     return "\\def\\ASYraw#1{#1}";
   else
@@ -1492,7 +1511,8 @@ const char *rawpostscript(const string& texengine) {
 }
 
 // TeX macro to begin picture
-const char *beginpicture(const string& texengine) {
+const char *beginpicture(const string& texengine) 
+{
   if(latex(texengine))
     return "\\begin{picture}";
   if(context(texengine))
@@ -1502,7 +1522,8 @@ const char *beginpicture(const string& texengine) {
 }
 
 // TeX macro to end picture
-const char *endpicture(const string& texengine) {
+const char *endpicture(const string& texengine)
+{
   if(latex(texengine))
     return "\\end{picture}%";
   else if(context(texengine))
@@ -1512,7 +1533,8 @@ const char *endpicture(const string& texengine) {
 }
 
 // Begin TeX special command.
-const char *beginspecial(const string& texengine) {
+const char *beginspecial(const string& texengine)
+{
   if(pdf(texengine))
     return xe(texengine) ? "\\special{pdf:literal " : "\\special{pdf:";
   else
@@ -1520,18 +1542,9 @@ const char *beginspecial(const string& texengine) {
 }
 
 // End TeX special command.
-const char *endspecial() {
-  return "}%";
-}
-
-bool fataltex[]={false,true};
-const char *pdftexerrors[]={"! "," ==> Fatal error",NULL};
-const char *texerrors[]={"! ",NULL};
-
-// Messages that signify a TeX error.
-const char **texabort(const string& texengine)
+const char *endspecial()
 {
-  return settings::pdf(texengine) ? pdftexerrors : texerrors;
+  return "}%";
 }
 
 string texcommand()
@@ -1558,11 +1571,11 @@ Int getScroll()
     if(terminal) {
       int error;
       error=setupterm(terminal,1,&error);
-#ifndef __CYGWIN__      
+#ifndef __MSDOS__      
       if(error == 0) scroll=lines > 2 ? lines-1 : 1;
       else
 #endif
-	scroll=0;
+        scroll=0;
     } else scroll=0;
 
   }
@@ -1588,7 +1601,7 @@ void setOptions(int argc, char *argv[])
   argv0=argv[0];
 
   cout.precision(DBL_DIG);
-  
+
   // Build settings module.
   initSettings();
   
@@ -1618,6 +1631,9 @@ void setOptions(int argc, char *argv[])
   // Read command-line options again to override configuration file defaults.
   getOptions(argc,argv);
   
+  if(getSetting<Int>("outpipe") == 2) // Redirect cerr to cout
+    std::cerr.rdbuf(std::cout.rdbuf());
+  
   Setting("sysdir")=sysdir;
   
   if(docdir.empty())
